@@ -2,29 +2,41 @@ from pyravendb.commands.raven_commands import GetTopologyCommand, GetStatisticsC
 from pyravendb.connection.requests_helpers import *
 from pyravendb.custom_exceptions import exceptions
 from OpenSSL import crypto
-from pyravendb.data.document_convention import DocumentConvention
-from threading import Lock, Thread
+from pyravendb.data.document_conventions import DocumentConventions
+from threading import Lock
 from pyravendb.tools.utils import Utils
 from datetime import datetime, timedelta
 import time
 import requests
 import json
 import hashlib
+import errno
 import os
-
+import io
 import logging
 
 logging.basicConfig(filename='requests_executor_info.log', level=logging.DEBUG)
 log = logging.getLogger()
 
+TOPOLOGY_FILES_DIR = os.path.join(os.getcwd(), "topology_files")
+
 
 class RequestsExecutor(object):
-    def __init__(self, database_name, certificate, convention=None, **kwargs):
+    def __new__(cls, *args, **kwargs):
+        instance = super(RequestsExecutor, cls).__new__(cls)
+        try:
+            os.makedirs(TOPOLOGY_FILES_DIR)
+        except OSError as e:
+            if e.errno != errno.EEXIST:
+                raise
+        return instance
+
+    def __init__(self, database_name, certificate, conventions=None, **kwargs):
         self._database_name = database_name
         self._certificate = certificate
         self.topology_etag = kwargs.get("topology_etag", 0)
         self._last_return_response = datetime.utcnow()
-        self.convention = convention if convention is not None else DocumentConvention()
+        self.conventions = conventions if conventions is not None else DocumentConventions()
         self._node_selector = kwargs.get("node_selector", None)
         self._last_known_urls = None
 
@@ -44,9 +56,13 @@ class RequestsExecutor(object):
 
         self._closed = False
 
+    @property
+    def certificate(self):
+        return self._certificate
+
     @staticmethod
-    def create(urls, database_name, certificate, convention=None):
-        executor = RequestsExecutor(database_name, certificate, convention)
+    def create(urls, database_name, certificate, conventions=None):
+        executor = RequestsExecutor(database_name, certificate, conventions)
         executor.start_first_topology_thread(urls)
         return executor
 
@@ -60,6 +76,17 @@ class RequestsExecutor(object):
         self._first_topology_update = PropagatingThread(target=self.first_topology_update, args=(urls,), daemon=True)
         self._first_topology_update.start()
 
+    def ensure_node_selector(self):
+        if self._first_topology_update and self._first_topology_update.is_alive():
+            self._first_topology_update.join()
+
+        if not self._node_selector:
+            self._node_selector = NodeSelector(Topology(etag=self.topology_etag, nodes=self.topology_nodes))
+
+    def get_preferred_node(self):
+        self.ensure_node_selector()
+        return self._node_selector.get_current_node()
+
     def execute(self, raven_command, should_retry=True):
         if not hasattr(raven_command, 'raven_command'):
             raise ValueError("Not a valid command")
@@ -100,12 +127,18 @@ class RequestsExecutor(object):
                 if not self._disable_topology_updates:
                     raven_command.headers["Topology-Etag"] = "\"{0}\"".format(self.topology_etag)
 
-                data = None if raven_command.data is None else \
-                    json.dumps(raven_command.data, default=self.convention.json_default_method)
+                data = raven_command.data
+                if data and not isinstance(data, io.BufferedIOBase):
+                    data = json.dumps(raven_command.data, default=self.conventions.json_default_method)
+
+                if raven_command.files:
+                    data = {"data": data}
+
                 start_time = time.time() * 1000
                 end_time = None
                 try:
                     response = session.request(raven_command.method, url=raven_command.url, data=data,
+                                               files=raven_command.files,
                                                cert=self._certificate, headers=raven_command.headers,
                                                stream=raven_command.use_stream)
                 except Exception as e:
@@ -139,7 +172,7 @@ class RequestsExecutor(object):
                     raise exceptions.AuthorizationException(
                         "Forbidden access to " + chosen_node.database + "@" + chosen_node.url + ", " +
                         ("a certificate is required." if self._certificate is None
-                        else name + " does not have permission to access it or is unknown.") +
+                         else name + " does not have permission to access it or is unknown.") +
                         response.request.method + " " + response.request.path_url)
                 if response.status_code == 410:
                     if should_retry:
@@ -211,7 +244,7 @@ class RequestsExecutor(object):
         server_hash = hashlib.md5(
             "{0}{1}".format(url, self._database_name).encode(
                 'utf-8')).hexdigest()
-        topology_file_path = "{0}\{1}.raven-topology".format(os.getcwd(), server_hash)
+        topology_file_path = os.path.join(TOPOLOGY_FILES_DIR, server_hash + ".raven-topology")
         try:
             with open(topology_file_path, 'r') as topology_file:
                 json_file = json.load(topology_file)
@@ -240,7 +273,7 @@ class RequestsExecutor(object):
                     "{0}{1}".format(node.url, node.database).encode(
                         'utf-8')).hexdigest()
 
-                topology_file = "{0}\{1}.raven-topology".format(os.getcwd(), hash_name)
+                topology_file = os.path.join(TOPOLOGY_FILES_DIR, hash_name + ".raven-topology")
                 try:
                     with open(topology_file, 'w') as outfile:
                         json.dump(response, outfile, ensure_ascii=False)
@@ -274,10 +307,10 @@ class RequestsExecutor(object):
                     self._failed_nodes_timers.update({chosen_node: node_status})
                     node_status.start_timer()
 
-            node_selector.on_failed_request(node_index)
-            current_node = node_selector.get_current_node()
-            if command.is_failed_with_node(current_node):
-                return False
+        node_selector.on_failed_request(node_index)
+        current_node = node_selector.get_current_node()
+        if command.is_failed_with_node(current_node):
+            return False
         return True
 
     def cancel_all_failed_nodes_timers(self):
