@@ -2,7 +2,10 @@ from pyravendb.tools.generate_id import GenerateEntityIdOnTheClient
 from pyravendb.store.session_query import Query
 from pyravendb.commands import commands_data
 from pyravendb.commands.raven_commands import *
+from pyravendb.raven_operations.operations import GetAttachmentOperation
+from pyravendb.commands.commands_data import PutAttachmentCommandData, DeleteAttachmentCommandData, DeleteCommandData
 from pyravendb.tools.utils import Utils
+from pyravendb.data.operation import AttachmentType
 
 
 class _SaveChangesData(object):
@@ -62,6 +65,15 @@ class DocumentSession(object):
     def documents_by_id(self):
         return self._documents_by_id
 
+    def defer(self, command, *args):
+        self._defer_commands.add(command)
+        for arg in args:
+            self._defer_commands.add(arg)
+
+    @property
+    def defer_commands(self):
+        return self._defer_commands
+
     @property
     def known_missing_ids(self):
         return self._known_missing_ids
@@ -225,7 +237,7 @@ class DocumentSession(object):
             return
         self._known_missing_ids.add(key_or_entity)
         self._included_documents_by_id.pop(key_or_entity, None)
-        self._defer_commands.add(commands_data.DeleteCommandData(key_or_entity, expected_change_vector))
+        self.defer(commands_data.DeleteCommandData(key_or_entity, expected_change_vector))
 
     def assert_no_non_unique_instance(self, entity, key):
         if not (key is None or key.endswith("/") or key not in self._documents_by_id
@@ -316,7 +328,7 @@ class DocumentSession(object):
                 if entity in self._documents_by_entity:
                     self._documents_by_id[item["@id"]] = entity
                     document_info = self._documents_by_entity[entity]
-                    document_info["change_vector"] = ["change_vector"]
+                    document_info["change_vector"] = item["@change-vector"]
                     item.pop("Type", None)
                     document_info["original_metadata"] = item.copy()
                     document_info["metadata"] = item
@@ -329,7 +341,6 @@ class DocumentSession(object):
             keys_to_delete.append(self._documents_by_entity[entity]["key"])
 
         for key in keys_to_delete:
-            existing_entity = None
             change_vector = None
             if key in self._documents_by_id:
                 existing_entity = self._documents_by_id[key]
@@ -338,7 +349,7 @@ class DocumentSession(object):
                         "@change-vector"] if self.advanced.use_optimistic_concurrency else None
                 self._documents_by_entity.pop(existing_entity, None)
                 self._documents_by_id.pop(key, None)
-            data.entities.append(existing_entity)
+                data.entities.append(existing_entity)
             data.commands.append(commands_data.DeleteCommandData(key, change_vector))
         self._deleted_entities.clear()
 
@@ -372,7 +383,7 @@ class DocumentSession(object):
                 "The maximum number of requests ({0}) allowed for this session has been reached. Raven limits the number \
                 of remote calls that a session is allowed to make as an early warning system. Sessions are expected to \
                 be short lived, and Raven provides facilities like batch saves (call save_changes() only once).\
-                You can increase the limit by setting DocumentConvention.\
+                You can increase the limit by setting DocumentConventions.\
                 MaxNumberOfRequestsPerSession or MaxNumberOfRequestsPerSession, but it is advisable \
                 that you'll look into reducing the number of remote calls first, \
                 since that will speed up your application significantly and result in a\
@@ -380,10 +391,39 @@ class DocumentSession(object):
 
 
 class Advanced(object):
+
     def __init__(self, session):
         self.session = session
         self.use_optimistic_concurrency = session.use_optimistic_concurrency if session.use_optimistic_concurrency \
             else session.conventions.use_optimistic_concurrency
+        self._attachment = None
+
+    @property
+    def attachment(self):
+        if not self._attachment:
+            self._attachment = _Attachment(self.session)
+        return self._attachment
+
+    def stream(self, query):
+        from pyravendb.store.stream import IncrementalJsonParser
+        import ijson
+
+        index_query = query.get_index_query()
+        if index_query.wait_for_non_stale_results:
+            raise exceptions.NotSupportedException("Since stream() does not wait for indexing (by design), "
+                                                   "streaming query with wait_for_non_stale_results is not supported.")
+        self.session.increment_requests_count()
+        command = QueryStreamCommand(index_query)
+        with self.session.requests_executor.execute(command) as response:
+            basic_parse = IncrementalJsonParser.basic_parse(response)
+            parser = ijson.backend.common.parse(basic_parse)
+
+            results = ijson.backend.common.items(parser, "Results")
+            for result in next(results, None):
+                document, metadata, _ = Utils.convert_to_entity(result, query.object_type, self.session.conventions,
+                                                                query.nested_object_types)
+                yield {"document": document, "metadata": metadata, "id": metadata.get("@id", None),
+                       "change-vector": metadata.get("@change-vector", None)}
 
     def number_of_requests_in_session(self):
         return self.session.number_of_requests_in_session
@@ -394,12 +434,11 @@ class Advanced(object):
                 return self.session.documents_by_entity[instance]["key"]
         return None
 
-    """
-    The document store associated with this session
-    """
-
     @property
     def document_store(self):
+        """
+        The document store associated with this session
+        """
         return self.session._document_store
 
     def clear(self):
@@ -408,3 +447,91 @@ class Advanced(object):
         self.session.deleted_entities.clear()
         self.session.included_documents_by_id.clear()
         self.session.known_missing_ids.clear()
+
+
+class _Attachment:
+    def __init__(self, session):
+        self._session = session
+        self._store = session.advanced.document_store
+        self._defer_commands = session.defer_commands
+
+    def _command_exists(self, document_id):
+        for command in self._defer_commands:
+            if not isinstance(command, (PutAttachmentCommandData, DeleteAttachmentCommandData, DeleteCommandData)):
+                continue
+
+            if command.key == document_id:
+                return command
+
+    def throw_not_in_session(self, entity):
+        raise ValueError(
+            repr(entity) + " is not associated with the session, cannot add attachment to it. "
+                           "Use documentId instead or track the entity in the session.")
+
+    def store(self, entity_or_document_id, name, stream, content_type=None, change_vector=None):
+        if not isinstance(entity_or_document_id, str):
+            entity = self._session.documents_by_entity.get(entity_or_document_id, None)
+            if not entity:
+                self.throw_not_in_session(entity_or_document_id)
+            entity_or_document_id = entity["metadata"]["@id"]
+
+        if not entity_or_document_id:
+            raise ValueError(entity_or_document_id)
+        if not name:
+            raise ValueError(name)
+
+        defer_command = self._command_exists(entity_or_document_id)
+        if defer_command:
+            message = "Can't store attachment {0} of document {1}".format(name, entity_or_document_id)
+            if defer_command.type == "DELETE":
+                message += ", there is a deferred command registered for this document to be deleted."
+            elif defer_command.type == "AttachmentPUT":
+                message += ", there is a deferred command registered to create an attachment with the same name."
+            elif defer_command.type == "AttachmentDELETE":
+                message += ", there is a deferred command registered to delete an attachment with the same name."
+            raise exceptions.InvalidOperationException(message)
+
+        entity = self._session.documents_by_id.get(entity_or_document_id, None)
+        if entity and entity in self._session.deleted_entities:
+            raise exceptions.InvalidOperationException(
+                "Can't store attachment " + name + " of document" + entity_or_document_id +
+                ", the document was already deleted in this session.")
+
+        self._session.defer(PutAttachmentCommandData(entity_or_document_id, name, stream, content_type, change_vector))
+
+    def delete(self, entity_or_document_id, name):
+        if not isinstance(entity_or_document_id, str):
+            entity = self._session.documents_by_entity.get(entity_or_document_id, None)
+            if not entity:
+                self.throw_not_in_session(entity_or_document_id)
+            entity_or_document_id = entity["metadata"]["@id"]
+
+        if not entity_or_document_id:
+            raise ValueError(entity_or_document_id)
+        if not name:
+            raise ValueError(name)
+
+        defer_command = self._command_exists(entity_or_document_id)
+        if defer_command:
+            if defer_command.type in ("Delete", "AttachmentDELETE"):
+                return
+            if defer_command.type == "AttachmentPUT":
+                raise exceptions.InvalidOperationException(
+                    "Can't delete attachment " + name + " of document " + entity_or_document_id +
+                    ",there is a deferred command registered to create an attachment with the same name.")
+
+        entity = self._session.documents_by_id.get(entity_or_document_id, None)
+        if entity and entity in self._session.deleted_entities:
+            return
+
+        self._session.defer(DeleteAttachmentCommandData(entity_or_document_id, name, None))
+
+    def get(self, entity_or_document_id, name):
+        if not isinstance(entity_or_document_id, str):
+            entity = self._session.documents_by_entity.get(entity_or_document_id, None)
+            if not entity:
+                self.throw_not_in_session(entity_or_document_id)
+            entity_or_document_id = entity["metadata"]["@id"]
+
+        operation = GetAttachmentOperation(entity_or_document_id, name, AttachmentType.document, None)
+        return self._store.operations.send(operation)
