@@ -1,8 +1,9 @@
 from pyravendb.connection.requests_executor import RequestsExecutor
 from pyravendb.custom_exceptions import exceptions
-from pyravendb.data.document_convention import DocumentConvention
+from pyravendb.data.document_conventions import DocumentConventions
 from pyravendb.hilo.hilo_generator import MultiDatabaseHiLoKeyGenerator
 from pyravendb.store.document_session import DocumentSession
+from pyravendb.changes.database_changes import DatabaseChanges
 from pyravendb.subscriptions.document_subscriptions import DocumentSubscriptions
 from threading import Lock
 import uuid
@@ -15,15 +16,17 @@ class DocumentStore(object):
             urls = [urls]
         self.urls = urls
         self.database = database
-        self.conventions = DocumentConvention()
+        self.conventions = DocumentConventions()
         self._certificate = certificate
         self._request_executors = {}
         self._initialize = False
         self.generator = None
         self._operations_executor = None
         self.lock = Lock()
+        self.add_change_lock = Lock()
         self._maintenance_operation_executor = None
         self.subscriptions = DocumentSubscriptions(self)
+        self._database_changes = {}
 
     @property
     def certificate(self):
@@ -47,9 +50,37 @@ class DocumentStore(object):
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
+
+    def close(self):
         if self.generator is not None:
             self.generator.return_unused_range()
 
+        if self.subscriptions:
+            self.subscriptions.close()
+
+        database_changes_keys = list(self._database_changes.keys())
+        for key in database_changes_keys:
+            self._database_changes[key].close()
+
+        for _, request_executor in self._request_executors.items():
+            request_executor.close()
+
+    def changes(self, database=None, on_error=None, executor=None):
+        self._assert_initialize()
+        if not database:
+            database = self.database
+        with self.add_change_lock:
+            if database not in self._database_changes:
+                self._database_changes[database] = DatabaseChanges(request_executor=self.get_request_executor(database),
+                                                                   database_name=database,
+                                                                   on_close=self._on_close_change, on_error=on_error,
+                                                                   executor=executor)
+            return self._database_changes[database]
+
+    def _on_close_change(self, database):
+        del self._database_changes[database]
+
     def get_request_executor(self, db_name=None):
         self._assert_initialize()
 
@@ -65,7 +96,7 @@ class DocumentStore(object):
     def initialize(self):
         if not self._initialize:
             if self.urls is None or len(self.urls) == 0:
-                raise ValueError("Document store URLs cannot be empty", "urks")
+                raise ValueError("Document store URLs cannot be empty", "urls")
             if self.database is None:
                 raise exceptions.InvalidOperationException("None database is not valid")
 
@@ -117,7 +148,7 @@ class MaintenanceOperationExecutor:
         except AttributeError:
             raise ValueError("Invalid operation")
 
-        command = operation.get_command(self.request_executor.convention)
+        command = operation.get_command(self.request_executor.conventions)
         return self.request_executor.execute(command)
 
 
@@ -145,7 +176,7 @@ class ServerOperationExecutor:
         except AttributeError:
             raise ValueError("Invalid operation")
 
-        command = operation.get_command(self.request_executor.convention)
+        command = operation.get_command(self.request_executor.conventions)
         return self.request_executor.execute(command)
 
 
@@ -188,5 +219,5 @@ class OperationExecutor(object):
         except AttributeError:
             raise ValueError("Invalid operation")
 
-        command = operation.get_command(self._store, self.request_executor.convention)
+        command = operation.get_command(self._store, self.request_executor.conventions)
         return self.request_executor.execute(command)
