@@ -1,5 +1,6 @@
 import logging
 import socket
+import ssl
 import sys
 from datetime import datetime
 from pyravendb.tools.utils import Utils
@@ -10,6 +11,7 @@ from pyravendb.subscriptions.data import IncrementalJsonParser
 from pyravendb.custom_exceptions.exceptions import *
 from pyravendb.connection.requests_executor import RequestsExecutor
 from pyravendb.custom_exceptions.exceptions import AggregateException
+import ijson
 
 
 class SubscriptionWorker:
@@ -82,9 +84,13 @@ class SubscriptionWorker:
         self._my_socket.settimeout(30)
 
         # TODO: wrap SSL
-        # ssl_socket = ssl.wrap_socket(sub_socket, certfile=self._store.certificate,
-        #                              ssl_version=ssl.PROTOCOL_TLSv1_2, do_handshake_on_connect=True)
-        self._my_socket.connect((host, port))
+        if self._store.certificate:
+            self._my_socket = ssl.wrap_socket(self._my_socket, certfile=self._store.certificate,
+                                              ssl_version=ssl.PROTOCOL_TLSv1_2)
+        try:
+            self._my_socket.connect((host, port))
+        except Exception as e:
+            print(e)
 
         header = Utils.dict_to_bytes(
             {"Operation": "Subscription", "DatabaseName": self._database_name, "OperationVersion": 40})
@@ -178,9 +184,10 @@ class SubscriptionWorker:
                 try:
                     self._process_documents(self._batch)
                 except Exception as ex:
-                    self._logger.info(
-                        "Subscription '{0}'. Subscriber threw an exception on document batch : {1}".format(
-                            self._options.subscription_name, str(ex)))
+                    if sys.gettrace():
+                        self._logger.info(
+                            "Subscription '{0}'. Subscriber threw an exception on document batch : {1}".format(
+                                self._options.subscription_name, str(ex)))
                     if not self._options.ignore_subscriber_errors:
                         raise SubscriberErrorException(
                             "Subscriber threw an exception in subscription" + self._options.subscription_name, ex)
@@ -219,7 +226,7 @@ class SubscriptionWorker:
                 SubscriptionInUseException, SubscriptionDoesNotExistException, SubscriptionClosedException,
                 SubscriptionInvalidStateException, DatabaseDoesNotExistException, AuthorizationException,
                 AllTopologyNodesDownException, SubscriberErrorException, ValueError, NotImplementedError,
-                AttributeError)):
+                AttributeError, ijson.backend.UnexpectedSymbol)):
             self.close()
             return False
         self.assert_last_connection_failure()
