@@ -2,7 +2,7 @@ from pyravendb.data.query import IndexQuery
 from pyravendb.tests.test_base import TestBase
 from pyravendb.commands.raven_commands import *
 from pyravendb.custom_exceptions.exceptions import ErrorResponseException
-from pyravendb.data.document_convention import DocumentConvention
+from pyravendb.data.document_conventions import DocumentConventions
 import time
 
 
@@ -22,14 +22,14 @@ class TestQuery(TestBase):
         query_command = QueryCommand(
             index_query=IndexQuery(query="From INDEX 'AllDocuments' WHERE Tag='Products'",
                                    wait_for_non_stale_results=True),
-            conventions=DocumentConvention())
+            conventions=DocumentConventions())
         response = self.requests_executor.execute(query_command)
         self.assertEqual(response["Results"][0]["Name"], "test")
 
     def test_get_only_metadata(self):
         query_command = QueryCommand(
             index_query=IndexQuery(query="From INDEX 'AllDocuments' WHERE Tag='Products'",
-                                   wait_for_non_stale_results=True), conventions=DocumentConvention(),
+                                   wait_for_non_stale_results=True), conventions=DocumentConventions(),
             metadata_only=True)
         response = self.requests_executor.execute(query_command)
         self.assertFalse("Name" in response["Results"][0])
@@ -37,7 +37,7 @@ class TestQuery(TestBase):
     def test_get_only_index_entries(self):
         query_command = QueryCommand(index_query=IndexQuery(query="FROM INDEX 'AllDocuments' WHERE Tag='products'",
                                                             wait_for_non_stale_results=True),
-                                     conventions=DocumentConvention(),
+                                     conventions=DocumentConventions(),
                                      index_entries_only=True)
 
         start = time.time()
@@ -50,12 +50,12 @@ class TestQuery(TestBase):
     def test_fail_with_no_index(self):
         query_command = QueryCommand(index_query=IndexQuery(query="From INDEX IndexIsNotExists WHERE Tag='products'",
                                                             wait_for_non_stale_results=True),
-                                     conventions=DocumentConvention())
+                                     conventions=DocumentConventions())
         self.assertIsNone(self.requests_executor.execute(query_command))
 
     def test_fail_with_wrong_query(self):
         with self.assertRaises(ErrorResponseException):
             query_command = QueryCommand(index_query=IndexQuery(query="From INDEX IndexIsNotExists WHERE Tag=products",
                                                                 wait_for_non_stale_results=True),
-                                         conventions=DocumentConvention())
+                                         conventions=DocumentConventions())
             self.assertIsNone(self.requests_executor.execute(query_command))
