@@ -5,6 +5,12 @@ from pyravendb.data.indexes import IndexDefinition, SortOptions, IndexFieldOptio
 import unittest
 
 
+class User:
+    def __init__(self, name, age):
+        self.name = name
+        self.age = age
+
+
 class Product(object):
     def __init__(self, name, key, order):
         self.name = name
